{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit FileWork;

interface

uses Windows, Sysutils, classes, Forms, Dialogs, menus, SyntaxEd, Files;

var
  DelBakFiles:boolean; //Delete bak Files
  HTMLDefaultExt:string;
  FlRestoreFile:boolean; //Don't Restore FIle
  ReOpenList:TStringList;
  ReOpenListMaxCnt:integer;

procedure DelBakFileName;
procedure RestoreFile;
function FileNameToBak(AFileName:string):string;
function FWCloseFile:boolean;
function CloseAllFiles:boolean;
function SaveChanges:boolean;
function HTMLMemoSaveToFileName:boolean;
function FWFileExists(AFileName:string):boolean;
function FWEditorLoadFile(AEditorEntry:TEditorEntry;AFileName:string):boolean;
function LoadFileName(AFileName:string):boolean;
procedure ReOpenListAdd(AStr:string);
procedure ReOpenListCrop;
function FWSaveDialogCanClose:boolean;
procedure FWSaveAs;
procedure FWNew;
procedure FWOpen;
procedure FWOpenText(AFileNum:integer);
procedure FWReOpenCreate;
procedure FWReOpenFileClick(AFileName:string);
procedure FWSaveAll;
procedure FWConvertText(ASender:TObject);
function FWGetFilesModified:boolean;
procedure FWCmpAgeFiles;
procedure FWDialogInit(ADialogObject:TCommonDialog;AInitDir,AFileName:string;ATitle:string;AMultiSel:boolean);
function GetParamStrs:string;

implementation

uses Main;

var
  FCloseAll:boolean;
  FCloseNoToAll:boolean;

function FileNameToBak(AFileName:string):string;
var
  XStr:string;
  XExt:string;
begin
  XExt:=ExtractFileExt(AFileName);
  if XExt=''
    then XExt:='.~'
    else Insert('~',XExt,2);
  XStr:=ChangeFileExt(AFileName,XExt);
  while FileExists(XStr) do Insert('_',XStr,length(XStr)-length(XExt)+1);
  result:=XStr;
  end;

procedure DelBakFileName;
begin
  if DelBakFiles then
    DeleteFile(FEdEntry.BakFileName);
  end;

procedure RestoreFile;
var bakfile:string;
begin
  bakfile:=FEdEntry.BakFilename;
  if FlRestoreFile then begin
    FlRestoreFile:=false;
    if not FileExists(bakfile) then begin
      MessageDlg('File cannot be restored.'+#13+#10+
                 '(File "'+bakfile+'" does not exist)',
                 mtError,[mbOK],0);
      exit;
      end;
    if DelBakFiles then begin
      DeleteFile(FEdEntry.FileName);
      RenameFile(bakfile, FEdEntry.FileName);
    end else CopyFile(PChar(bakfile),PChar(FEdEntry.FileName),false);
  end else DelBakFileName;
  end;

function SaveChanges:boolean;
var
  code:integer;
  PromtToSave:string;
  XButtons:TMsgDlgButtons;

  function ModifiedOnDisk:boolean;
  var XBakFileAge,XFileAge:integer;
  begin
    result:=false;
    if Assigned(FEdEntry) then begin
      XBakFileAge:=FileAge(FEdEntry.BakFilename);
      XFileAge:=FileAge(FEdEntry.FileName);
      if XFileAge=-1 then begin
        PromtToSave:='File "'+FEdEntry.FileName+'" has been deleted on disk.';
        result:=true;
      end else if XBakFileAge<>XFileAge then begin
        PromtToSave:='File "'+FEdEntry.FileName+'" has been modified on disk.';
        result:=true;
        end;
      end;
    end;

begin
  result:=true;
  PromtToSave:='File "'+FEdEntry.FileName+'" has been modified in memory.';
  if Assigned(FEdEntry) and
    (FEdEntry.Editor.Modified or ModifiedOnDisk)
    then begin
    if FCloseNoToAll then begin
      FlRestoreFile:=true;
      exit;
      end;
    XButtons:=[mbYes,mbNo,mbCancel];
    if FCloseAll and (MainForm.FileTabs.Tabs.Count>1) then XButtons:=XButtons+[mbNoToAll];
    code:=MessageDlg(PromtToSave + #13 +
      'Save changes?',mtConfirmation,XButtons,0);
    if (code=IDYES) then result:=HTMLMemoSaveToFileName else
    if (code=IDCANCEL) then result:=false else
    if (code=IDNO) then FlRestoreFile:=true else begin
      FlRestoreFile:=true;
      FCloseNoToAll:=true; //NoToAll
      end;
    end;
  end;

function FWCloseFile:boolean;
//if false then exit;
begin
  result:=false;
  if Assigned(FEdEntry) and (not SaveChanges) then exit;
  RestoreFile;
  Files.CloseTab;
  result:=true;
  end;

function CloseAllFiles:boolean;
//if false then exit;
begin
  result:=true;
  FCloseAll:=true;
  while Assigned(FEdEntry) do
    if not FWCloseFile then begin
      result:=false;
      break;
      end;
  FCloseAll:=false;
  FCloseNoToAll:=false;
  end;

function FWFileExists(AFileName:string):boolean;
begin
  result:=false;
  if FileExists(AFileName) then result:=true
  else begin
    MessageDlg('File "'+AFileName+'" does not exist.', mtError, [mbOk], 0);
    exit;
    end;
  end;

function FWEditorLoadFile(AEditorEntry:TEditorEntry;AFileName:string):boolean;
begin
  result:=false;
  if Assigned(AEditorEntry) and FWFileExists(AFileName) then
    try
      AEditorEntry.Editor.LoadFromFile(AFileName);
      AEditorEntry.AgeFile:=FileAge(AFileName);
      AEditorEntry.FileName:=AFileName;
      result:=true;
    except
      MessageDlg('File "'+AFileName+'" cannot be opened.', mtError, [mbOk], 0);
      end;
  end;

function RestoreLongName(fn: string): string;
function LookupLongName(const filename: string): string;
var sr: TSearchRec;
begin
  if FindFirst(filename, faAnyFile, sr)=0 then Result:=sr.Name
                                          else Result:=ExtractFileName(filename);
  SysUtils.FindClose(sr);
  end;
function GetNextFN: string;
var i: integer;
begin
  Result:='';
  if Pos('\\', fn)=1 then begin
    Result:='\\';
    fn:=Copy(fn, 3, length(fn)-2);
    i:=Pos('\', fn);
    if i<>0 then begin
      Result:=Result+Copy(fn,1,i);
      fn:=Copy(fn, i+1, length(fn)-i);
      end;
    end;
  i:=Pos('\', fn);
  if i<>0 then begin
    Result:=Result+Copy(fn,1,i-1);
    fn:=Copy(fn, i+1, length(fn)-i);
  end else begin
    Result:=Result+fn;
    fn:='';
    end;
  end;
var name: string;
begin
  if fn='' then begin
    result:='';
    exit;
    end;
  fn:=ExpandFileName(fn);
  Result:=GetNextFN;
  repeat
    name:=GetNextFN;
    Result:=Result+'\'+LookupLongName(Result+'\'+name);
    until length(fn)=0;
  end;

function LoadFileName(AFileName:string):boolean;
begin
  result:=false;
  AFileName:=RestoreLongName(AFileName);
  if (AFileName<>'') and FWFileExists(AFileName) then begin
    if (not Files.AddTab(AFileName))and
       (Assigned(FEdEntry)and  //For example if FNew when AFileName=FEdEntry.FileName
        (FEdEntry.AgeFile=FileAge(AFileName))) then exit;
    if Assigned(FEdEntry) and FWEditorLoadFile(FEdEntry,AFileName) then begin
      FEdEntry.BakFileName:=FileNameToBak(AFileName);
      CopyFile(PChar(AFileName),PChar(FEdEntry.BakFileName),false);
      ReOpenListAdd(AFileName);
      MainForm.HTMLMemoChange(nil);
      MainForm.RefreshTimer.Enabled:=false;
      MainForm.SetMainFormCaption;
    end else begin
      Files.CloseTab;
      exit;
      end;
    result:=true;
    end
  end;

procedure ReOpenListAdd(AStr:string);
begin
  with ReOpenList do
    if IndexOf(AStr)=-1 then begin
      Insert(0,AStr);
      if Count>ReOpenListMaxCnt then
        ReOpenListCrop;
    end else begin
      Delete(IndexOf(AStr));
      ReOpenListAdd(AStr); // :)
      end;
  end;

procedure ReOpenListCrop;
begin
  with ReOpenList do
    while Count>ReOpenListMaxCnt do Delete(Count-1);
  end;

function FWEditorEntrySaveFile(AEditorEntry:TEditorEntry):boolean;
var XMemLineStyle :TSelLineStyle;
  function CheckAccess(AFileName:string):boolean;
  var XHandle:THandle;
  begin
    result:=false;
    XHandle:=CreateFile(PChar(AFileName), GENERIC_WRITE,
      0, nil, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
    if XHandle <> INVALID_HANDLE_VALUE then begin
      CloseHandle(XHandle);
      result:=true;
      end;
    end;
begin
  result:=false;
  if Assigned(AEditorEntry) and Assigned(AEditorEntry.Editor) then begin
    if not CheckAccess(AEditorEntry.FileName) then exit;
    with AEditorEntry.Editor do begin
      XMemLineStyle:=SelLineStyle;
      SelLineStyle:=lsCRLF;
      KeepUndoOnSaveFile := MainForm.OUndoAfterSave.Checked;
      Lines.SaveToFile(AEditorEntry.FileName);
      SelLineStyle:=XMemLineStyle;
      AEditorEntry.AgeFile:=FileAge(AEditorEntry.FileName);
      end;
    result:=true;
    end;
  end;

function HTMLMemoSaveToFileName:boolean;
//result for cancel
var
  XButtons:TMsgDlgButtons;
  code:integer;
begin
  result:=true;
  if Assigned(FEdEntry) and FEdEntry.Editor.Modified then begin
    XButtons:=[mbYes,mbNo];
    if FCloseAll then XButtons:=XButtons+[mbCancel];
    while not FWEditorEntrySaveFile(FEdEntry) do begin
      code:=MessageDlg('File "'+FEdEntry.FileName+'" cannot be saved now.'#13'Do you want to try again?',
                       mtError, XButtons, 0);
      if code=idYes then continue;
      if code=idNo then exit;
      if code=idCancel then begin
        result:=false;
        exit;
        end;
      end;
    end;
  end;

function FWSaveDialogCanClose:boolean;
var
  CloseFileName:string;
begin
  result:=true;
  CloseFileName:=MainForm.SaveDialog1.FileName;
  if ExtractFileExt(CloseFileName)='' then
    CloseFileName:=ChangeFileExt(CloseFileName,'.'+FileWork.HTMLDefaultExt);
  if FileExists(CloseFileName) and
    (MessageDlg(CloseFileName+#13+'already exists.'+#13+#13+
      'Do you want to replace it?',
      mtWarning, [mbYes,mbNo], 0) <> idYes) then
    result:=false;
  end;

procedure FWSaveAs;
begin
  if Assigned(FEdEntry) then begin
    FWDialogInit(MainForm.SaveDialog1,'','','Save as',false);
    if MainForm.SaveDialog1.Execute then begin
      if not SaveChanges then exit;
      RestoreFile;
      FEdEntry.FileName:=MainForm.SaveDialog1.FileName;
      if ExtractFileExt(FEdEntry.FileName)='' then
        FEdEntry.FileName:=ChangeFileExt(FEdEntry.FileName,'.'+FileWork.HTMLDefaultExt);
      FWEditorEntrySaveFile(FEdEntry); //Doesn't cmp Modified
      FEdEntry.BakFileName:=FileNameToBak(FEdEntry.FileName);
      CopyFile(PChar(FEdEntry.FileName),PChar(FEdEntry.BakFileName),false);
      Files.ChangeFileName;
      MainForm.toBrowserToolButtonClick(nil);
      end;
    end;
  end;

procedure FWNew;
var
  LdFileName:string;
begin
  with MainForm do begin
    FWDialogInit(SaveDialog1,'','','New file',false);
    if SaveDialog1.Execute then begin
      LdFileName:=SaveDialog1.FileName;
      if ExtractFileExt(LdFileName)='' then
        LdFileName:=ChangeFileExt(LdFileName,'.'+FileWork.HTMLDefaultExt);
      CopyFile(PChar(ExtractFilePath(Application.ExeName) + 'default.html'),
        PChar(LdFileName),false);
      FindAddressLoad(LdFileName);
      end;
    end;
  end;

procedure FWOpen();
var i:integer;
begin
  with MainForm do begin
    FWDialogInit(OpenDialog1,'','','Open',true);
    if OpenDialog1.Execute then
      for i:=OpenDialog1.Files.Count-1 downto 0 do
        FindAddressLoad(ExpandFileName(OpenDialog1.Files[i]));
    end;
  end;

procedure FWOpenText(AFileNum:integer);
//if AFileNum is incorrect then InitDir and FileName are default
var
  i:integer;
  XFileName:string;
begin
  with MainForm do begin
    if (AFileNum >= 0) and (AFileNum < FileTabs.Tabs.Count) then begin
      XFileName:=TEditorEntry(FileTabs.Tabs.Objects[AFileNum]).FileName;
      FWDialogInit(OpenDialog1,ExtractFileDir(XFileName),XFileName,'Open Text',true);
    end else
      FWDialogInit(OpenDialog1,'','','Open Text',true);
    if OpenDialog1.Execute then
      for i:=OpenDialog1.Files.Count-1 downto 0 do
        LoadFileName(ExpandFileName(OpenDialog1.Files[i]));
    end;
  end;

procedure FWReOpenCreate;
var
  NewItem: TMenuItem;
  i : integer;
begin
  MainForm.FReOpen.Enabled:=true;
  MainForm.FReOpen.Clear;
  for i := 0 to ReOpenList.Count-1 do begin
    NewItem := TMenuItem.Create(nil);
    if i<=9 then NewItem.Caption := '&'+IntToStr(i)+' '
    else NewItem.Caption := '&'+char(i+55)+' ';
    NewItem.Caption := NewItem.Caption + ReOpenList.Strings[i];
    NewItem.OnClick:=MainForm.FReOpenFileOnClick;
    MainForm.FReOpen.Add(NewItem);
    end
  end;

procedure FWReOpenFileClick(AFileName:string);
begin
  MainForm.FindAddressLoad(Copy(AFileName,4,Length(AFileName)));
  end;

procedure FWSaveAll;
var i:integer;
begin
  if Assigned(FEdEntry) then
    with MainForm.FileTabs do
      for i:=0 to Tabs.Count-1 do
        if TEditorEntry(Tabs.Objects[i]).Editor.Modified then
          FWEditorEntrySaveFile(TEditorEntry(Tabs.Objects[i]));
  end;

procedure FWConvertText(ASender:TObject);
begin
  if Assigned(FEdEntry) then
    with MainForm do begin
      FWDialogInit(SaveDialog1,'','','Convert Text',false);
      if SaveDialog1.Execute then begin
        if ASender=FConvertTextHTML then
          FEdEntry.Editor.SaveFormat := sfHTML;
        if ASender=FConvertTextRTF then
          FEdEntry.Editor.SaveFormat := sfRTF;
        FEdEntry.Editor.KeepUndoOnSaveFile := MainForm.OUndoAfterSave.Checked;
        FEdEntry.Editor.SaveToFile(SaveDialog1.FileName);
        FEdEntry.Editor.SaveFormat := sfText;
        end;
      end;
  end;

function FWGetFilesModified:boolean;
var i:integer;
begin
  result:=true;
  if Assigned(FEdEntry) then
    with MainForm.FileTabs do
      for i:=0 to Tabs.Count-1 do
        if TEditorEntry(Tabs.Objects[i]).Editor.Modified then exit;
  result:=false;
  end;

procedure FWCmpAgeFiles;
var i:integer;
begin
  if Assigned(FEdEntry) then
    with MainForm.FileTabs do
      for i:=0 to Tabs.Count-1 do
        if FileExists(TEditorEntry(Tabs.Objects[i]).FileName) and
           (FileAge(TEditorEntry(Tabs.Objects[i]).FileName)<>
            TEditorEntry(Tabs.Objects[i]).AgeFile)and
           (MessageDlg('File "'+TEditorEntry(Tabs.Objects[i]).FileName+
                        '" has been modified on disk. Reload it?',
                        mtWarning, [mbYes, mbNo], 0) = IDYES)
          then begin
            FWEditorLoadFile(TEditorEntry(Tabs.Objects[i]),
                             TEditorEntry(Tabs.Objects[i]).FileName);
            if i=TabIndex then MainForm.HTMLMemoChange(nil);
          end else begin
            TEditorEntry(Tabs.Objects[i]).AgeFile:=
              FileAge(TEditorEntry(Tabs.Objects[i]).FileName);
            end;
  end;

procedure FWDialogInit(ADialogObject:TCommonDialog;AInitDir,AFileName:string;ATitle:string;AMultiSel:boolean);
begin
  //ForceCurrentDirectory:=false;
  with (ADialogObject as TOpenDialog) do begin
    Title:=ATitle;
    if AMultiSel
      then Options:=Options+[ofAllowMultiSelect]
      else Options:=Options-[ofAllowMultiSelect];
    if (AInitDir<>'')and(AFileName<>'') then begin
      InitialDir:=AInitDir;
      FileName:=AFileName;
    end else
      if Assigned(FEdEntry) then begin
        InitialDir:=ExtractFileDir(FEdEntry.FileName);
        FileName:=FEdEntry.FileName;
      end else begin
        InitialDir:='';
        FileName:='';
        end;
    end;
  end;

function GetParamStrs:string;
var XStr:string;
begin
  XStr:=CmdLine;
  if XStr[1]='"'
    then delete(XStr,1,length(paramstr(0))+2)
    else delete(XStr,1,length(paramstr(0)));
  XStr:=Trim(XStr);
  if (XStr<>'') and (XStr[1]='"') then result:=paramstr(1)
  else result:=XStr;
  end;

end.
