{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit Files;

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms,
  Main, SyntaxEd;
type
 TEditorEntry = class // Collection of data about one editor and its file / text
   private
     FEditor      : TSyntaxMemo;
     FFilename    : string;
     FBakFilename : string;
     FAgeFile     : integer;
   public
     constructor Create(aOwner: TComponent);
     destructor  Destroy; override;
     property  Editor      : TSyntaxMemo read FEditor;
     property  FileName    : string      read FFilename    write FFilename;
     property  BakFileName : string      read FBakFilename write FBakFilename;
     property  AgeFile     : integer     read FAgeFile     write FAgeFile;
   end;

var
  FFirst:boolean;
  DefParser:integer;
  FDoNotShowFocus:boolean;
  FEdEntry:TEditorEntry; //Current EditorEntry

function AddTab(aFilename: string):boolean;
procedure FileTabsChange;
function FileTabsChanging:boolean;
procedure CloseTab;
procedure SetTabNum(ANum:integer);
procedure ChangeAssignedHTMLMemo;
procedure ShowEditorFocus(AEditor:TSyntaxMemo;AFocused:boolean);
procedure ChangeFileName;

implementation

constructor TEditorEntry.Create(aOwner: TComponent);
begin
  inherited Create;
  FEditor := TSyntaxMemo.Create(aOwner);
  with FEditor do begin
    Visible := false;
    Parent  := aOwner as TWinControl;
    Width   := Parent.Width;
    Height  := Parent.Height;
    align   := alClient;
    BorderStyle := bsNone;
    HyperCursor   := crHandPoint;
    HideSelection := false;
    SelLineStyle  := lsLF; //for correct work with SelStart (Alt + Left/Right)
    Text    := '';
    end;
  end;

destructor  TEditorEntry.Destroy;
begin
  FEditor.Free;
  inherited Destroy;
  end;

function AddTab(aFilename: string):boolean;
var
  newEd : TEditorEntry;
  i,EqualNameNum:integer;
begin
  result:=false;
  with MainForm do begin
    Files.FileTabsChanging;
    if Assigned(FEdEntry) and (FEdEntry.FileName=AFileName) then begin
      Files.FileTabsChange;
      exit;
      end;
    EqualNameNum:=-1;
    for i:=0 to FileTabs.Tabs.Count-1 do
      if TEditorEntry(FileTabs.Tabs.Objects[i]).FileName=AFileName then begin
        EqualNameNum:=i;
        break;
        end;
    if EqualNameNum<>-1 then begin
      FileTabs.TabIndex:=EqualNameNum;
      Files.FileTabsChange;
    end else begin
      newEd := TEditorEntry.Create(EditorPanel);
      newEd.Editor.Parser1             := HTMLParser;
      newEd.Editor.Parser2             := PlainTextParser;
      newEd.Editor.Parser3             := SQLParser;
      newEd.Editor.ActiveParser        := DefParser;
      newEd.Editor.PopupMenu           := EditorPopupMenu;
      newEd.Editor.OnChange            := HTMLMemoChange;
      newEd.Editor.OnEnter             := HTMLMemoChangeFocus;
      newEd.Editor.OnExit              := HTMLMemoChangeFocus;
      newEd.Editor.fn_CharIsInWord     := HTMLMemofn_CmpChar;
      newEd.Editor.fn_CharStartsWord   := HTMLMemofn_CmpChar;
      newEd.Editor.OnCommandRecognised := HTMLMemoCommandRecognised;
      FileTabs.Tabs.InsertObject(FileTabs.TabIndex+1,StringReplace(ExtractFileName(aFilename),'&','&&',[rfReplaceAll]),newEd);
      FileTabs.TabIndex:=FileTabs.TabIndex+1;
      newEd.Editor.BringToFront;
      Files.FileTabsChange;
      if FileTabs.Tabs.Count=1 then ChangeAssignedHTMLMemo;
      result:=true;
      end;
    end;
  end;

procedure ShowEditorFocus(AEditor:TSyntaxMemo;AFocused:boolean);
begin
  if Assigned(AEditor) then
    if AFocused
      then AEditor.def_BackColor := clWhite
      else AEditor.def_BackColor := clBtnFace;
  end;

procedure FileTabsChange;
var
  XEd : TEditorEntry;
  memFocused : boolean;
begin
  with MainForm do begin
    if FileTabs.TabIndex = FileTabs.Tag then exit;
    FDoNotShowFocus:=true;
    memFocused:=Assigned(FEdEntry) and FEdEntry.Editor.Focused;
    FEdEntry := TEditorEntry(FileTabs.Tabs.Objects[FileTabs.TabIndex]);
    if Assigned(FEdEntry) then begin
      ShowEditorFocus(FEdEntry.Editor,memFocused);
      FEdEntry.Editor.Visible := true;
      end;
    if (FileTabs.Tag <> -1)and(FileTabs.Tag < FileTabs.Tabs.Count) then begin
      XEd := TEditorEntry(FileTabs.Tabs.Objects[FileTabs.Tag]);
      if Assigned(XEd.Editor) then XEd.Editor.Visible := false;
      end;
    FFirst:=true;
    HTMLMemoChange(nil);
    if Assigned(FEdEntry) and memFocused then HTMLMemoSetFocus
    else WBSetFocus;
    SetMainFormCaption;
    FDoNotShowFocus:=false;
    end;
  end;

function FileTabsChanging:boolean;
begin
  with MainForm do FileTabs.Tag := FileTabs.TabIndex;
  result:=true;
  end;

procedure CloseTab;
var
  oti : longint;
  XEditorEntry:TEditorEntry;
begin
  with MainForm do begin
    if Assigned(FEdEntry) then begin
      oti := FileTabs.TabIndex;
      XEditorEntry:=TEditorEntry(FileTabs.Tabs.Objects[oti]);
      HTMLMemoNotification(XEditorEntry.Editor,opRemove);
      FileTabs.Tabs.Delete(FileTabs.TabIndex);
      if (oti=0)and(FileTabs.Tabs.Count>0)
        then FileTabs.TabIndex := oti
        else FileTabs.TabIndex := oti - 1;
      if FileTabs.Tabs.Count = 0 then begin
        FEdEntry := nil;
        WBSetFocus;
        ChangeAssignedHTMLMemo;
        SetMainFormCaption;
      end else begin
        FileTabs.Tag := -1;
        Files.FileTabsChange;
        end;
      XEditorEntry.Free;
      end;
    end;
  end;

procedure SetTabNum(ANum:integer);
begin
  with MainForm do
    if ANum<=FileTabs.Tabs.Count then begin
      Files.FileTabsChanging;
      FileTabs.TabIndex:=ANum-1;
      Files.FileTabsChange;
      end;
  end;

procedure ChangeAssignedHTMLMemo;
var
  hs:boolean;
begin
  with MainForm do begin
    hs:=Assigned(FEdEntry);
    MEdit.Enabled    := hs;
    MSearch.Enabled  := hs;
    toBrowserToolButton.Enabled := hs;
    CloseToolButton.Enabled     := hs;
    if not hs then HTMLMemoChange(nil);
    end;
  end;

procedure ChangeFileName;
begin
  with MainForm do
    if Assigned(FEdEntry)and(FileTabs.TabIndex<>-1) then begin
      FileTabs.Tabs[FileTabs.TabIndex]:=StringReplace(ExtractFileName(FEdEntry.FileName),'&','&&',[rfReplaceAll]);
      SetMainFormCaption;
      end;
  end;

end.
