{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit FindDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, SyntaxEd;

type
  TEdFindDlg = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    FindHistoryBox: TComboBox;
    OptionsGroupBox: TGroupBox;
    UseCaseBox: TCheckBox;
    WholeWordBox: TCheckBox;
    RegExBox: TCheckBox;
    OKButton: TButton;
    CancelButton: TButton;
    DirectionGroupBox: TGroupBox;
    FwdDirection: TRadioButton;
    BackDirection: TRadioButton;
    ScopeGroupBox: TGroupBox;
    OriginGroupBox: TGroupBox;
    GlobalScope: TRadioButton;
    SelectedScope: TRadioButton;
    CursorOrigin: TRadioButton;
    EntireOrigin: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FwdDirectionClick(Sender: TObject);
    procedure BackDirectionClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FindHistoryBoxChange(Sender: TObject);
    procedure GlobalScopeClick(Sender: TObject);
    procedure SelectedScopeClick(Sender: TObject);
    procedure RegExBoxClick(Sender: TObject);
  private
    { Private declarations }
    FStartPoint        : longint;
    FOriginalSelection : TChRange;
    FFindText          : string;
    FEditor            : TSyntaxMemo;
    FModalResult       : integer;
    FLastFindText              : string;
    procedure UpdateOptionBoxes;
    procedure ReadOptionsBoxes;
  public
    { Public declarations }
    FFindOpt           : longint;
    function Execute(FindText: string; Options: Integer):integer;
    procedure DoFind(anEditor: TSyntaxMemo);
  end;

var
  EdFindDlg: TEdFindDlg;

implementation

{$R *.DFM}

{ TEdFindDlg }

function TEdFindDlg.Execute(FindText: string; Options: Integer):integer;
begin
  FFindText := FindText;
  FFindOpt  := Options;
  FLastFindText     := '';
  ShowModal;
  result := FModalResult;
  end;

procedure TEdFindDlg.UpdateOptionBoxes;
begin
  UseCaseBox.Checked    := FFindOpt and ft_MATCHCASE <> 0;
  WholeWordBox.Checked  := FFindOpt and ft_WHOLEWORD <> 0;
  RegExBox.Checked      := FFindOpt and ft_REGEXPR   <> 0;
  FwdDirection.Checked  := FFindOpt and ft_REVERSE   =  0;
  BackDirection.Checked := not (FwdDirection.Checked);
  end;

procedure TEdFindDlg.ReadOptionsBoxes;
begin
  FFindOpt := 0;
  if UseCaseBox.Checked    then FFindOpt := FFindOpt or ft_MATCHCASE;
  if WholeWordBox.Checked  then FFindOpt := FFindOpt or ft_WHOLEWORD;
  if RegExBox.Checked      then FFindOpt := FFindOpt or ft_REGEXPR;
  if BackDirection.Checked then FFindOpt := FFindOpt or ft_REVERSE;
  end;

procedure TEdFindDlg.FormShow(Sender: TObject);
begin
  if FindHistoryBox.Items.IndexOf(FFindText) = -1 then FindHistoryBox.Items.Add(FFindText);
  FindHistoryBox.ItemIndex := FindHistoryBox.Items.IndexOf(FFindText);
  UpdateOptionBoxes;
  FindHistoryBoxChange(Self);
  FindHistoryBox.SetFocus;
  end;

procedure TEdFindDlg.DoFind(anEditor: TSyntaxMemo);
var FoundAt : longint;
begin
  anEditor.Perform(SEM_SELECTION, eaGET, longint(@FOriginalSelection));
  FOriginalSelection := Normalise(FOriginalSelection);
  if GlobalScope.Checked then
    if EntireOrigin.Checked then
      if FwdDirection.Checked
        then FStartPoint := 0
        else FStartPoint := Length(anEditor.Lines.Text)-1
    else FStartPoint := FOriginalSelection.chStart
  else //Selected text
    if EntireOrigin.Checked then
      if FwdDirection.Checked
        then FStartPoint := FOriginalSelection.chStart //Forward
        else FStartPoint := FOriginalSelection.chEnd   //Backward
    else FStartPoint := FOriginalSelection.chStart; //From cursor
  with anEditor do begin
    if FLastFindText<>'' then
      if (UseCaseBox.Checked and (SelText = FLastFindText))
         or ((not UseCaseBox.Checked) and (Lowercase(SelText) = Lowercase(FLastFindText)))
      then
        if FwdDirection.Checked
          then FStartPoint := FStartPoint + 1
          else FStartPoint := FStartPoint - 1;
    anEditor.Perform(EM_SETSEL, FStartPoint, FStartPoint);
    if FindText(FindHistoryBox.Text, FoundAt, FFindOpt or ft_SILENT)
      and ( (not SelectedScope.Checked)
             or (SelectedScope.Checked and
              ((FoundAt >= FOriginalSelection.chStart)
                and (FoundAt+length(FindHistoryBox.Text) <= FOriginalSelection.chEnd)))
           )
    then begin
      if sizeof(TSM_char) = sizeof(WideChar) then
        if RegExBox.Checked
          then Perform(EM_SETSEL, CaretPosition.ByteOffsetFromCharOffset[FoundRESections[0]], CaretPosition.ByteOffsetFromCharOffset[FoundRESections[FoundRENumSections]])
          else Perform(EM_SETSEL, CaretPosition.ByteOffsetFromCharOffset[FoundAt],            CaretPosition.ByteOffsetFromCharOffset[FoundAt + length(FindHistoryBox.Text)])
      else
        if RegExBox.Checked
          then Perform(EM_SETSEL, FoundRESections[0], FoundRESections[FoundRENumSections])
          else Perform(EM_SETSEL, FoundAt,            FoundAt + length(FindHistoryBox.Text));
      FStartPoint := FoundAt;
      FLastFindText := SelText;
      end
    else begin
      //SelLength:=0;
      MessageDlg(format('"%s" not found', [FindHistoryBox.Text]),mtInformation,[mbOk],0);
      end;
    end;
  end;

procedure TEdFindDlg.OKButtonClick(Sender: TObject);
begin
  ReadOptionsBoxes; // Initiate the find...
  if FindHistoryBox.Items.IndexOf(FindHistoryBox.Text) = -1 then
    FindHistoryBox.Items.Insert(0, FindHistoryBox.Text);
  FModalResult := mrOK;
  Close;
  end;

procedure TEdFindDlg.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then begin
    FModalResult := mrCancel;
    Close;
    end;
  end;

procedure TEdFindDlg.FwdDirectionClick(Sender: TObject);
begin
  with (Sender as TRadioButton) do
    if not Checked then begin
      Checked := true;
      BackDirection.Checked := false;
      end;
  end;

procedure TEdFindDlg.BackDirectionClick(Sender: TObject);
begin
  with (Sender as TRadioButton) do
    if not Checked then begin
      Checked := true;
      FwdDirection.Checked := false;
      end;
  end;

procedure TEdFindDlg.CancelButtonClick(Sender: TObject);
begin
  FModalResult := mrCancel;
  Close;
  end;

procedure TEdFindDlg.FindHistoryBoxChange(Sender: TObject);
begin
  OKButton.Enabled := FindHistoryBox.Text <> '';
  end;

procedure TEdFindDlg.RegExBoxClick(Sender: TObject);
begin
  UseCaseBox.Enabled:=not RegExBox.Checked;
  WholeWordBox.Enabled:=not RegExBox.Checked;
  end;

procedure TEdFindDlg.GlobalScopeClick(Sender: TObject);
begin
  CursorOrigin.Enabled:=true;
  end;

procedure TEdFindDlg.SelectedScopeClick(Sender: TObject);
begin
  EntireOrigin.Checked:=true;
  CursorOrigin.Enabled:=false;
  end;

end.
