{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit HelpSearch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TSetHelpForm = class(TForm)
    OpenDialog1: TOpenDialog;
    TagPanel: TPanel;
    NoFindMemo: TMemo;
    Splitter1: TSplitter;
    FindMemo: TMemo;
    Panel1: TPanel;
    FindLabel: TLabel;
    ProgressBar1: TProgressBar;
    WoHTTPCheckBox: TCheckBox;
    OpenButton: TButton;
    SaveButton: TButton;
    CancelButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    { Private declarations }
    IndexFileName:string;
    FlExit:boolean;
  public
    { Public declarations }
  end;

var
  SetHelpForm: TSetHelpForm;

implementation

{$R *.DFM}

procedure TSetHelpForm.OpenButtonClick(Sender: TObject);
var
  FoundAt: LongInt;
  Str,GtStr:string;
  AtxtAt,AhrefAt:integer;
  Atxt,Ahref,ATag:string;
  BegSize:integer;
  TagCount:integer;
  HrefList,TagList:TStringList;

  function GetHref:string;
  var i:integer;
  begin
    for i:=AhrefAt to Length(GtStr) do
      if GtStr[i]='=' then break;
    AhrefAt:=i+1;
    for i:=AhrefAt to Length(GtStr) do
      if (GtStr[i]<>#32) and (GtStr[i]<>'"') and (GtStr[i]<>'''') then break;
    AhrefAt:=i;
    for i:=AhrefAt to Length(GtStr) do
      if (GtStr[i]=#32) or (GtStr[i]='"') or (GtStr[i]='''') then break;
    GetHref:=Copy(GtStr,AhrefAt,i-AhrefAt);
    end;

  function FindTag:string;
  var
    i,j,XPos:integer;
    FndStr:string;
  begin
    result:='';
    for j:=0 to TagList.Count-1 do begin
      FndStr:=Atxt;
      i:=1;
      XPos:=Pos(TagList[j],Copy(FndStr,i,Length(FndStr)-(i-1)));
      while XPos<>0 do
        if ((XPos-1+i=1)or(FndStr[XPos-2+i]<#65))and
           ((XPos-1+i-1+Length(TagList[j])=Length(FndStr))or
             (FndStr[XPos-1+i+Length(TagList[j])]<#65)) then begin
          result:=TagList[j];
          TagList.Delete(j);
          exit;
          end
        else begin
          i:=i+XPos;
          XPos:=Pos(TagList[j],Copy(FndStr,i,Length(FndStr)-i+1));
          end;
      end;
    end;

  procedure DelTagSpaces;
  var i:integer;
  begin
    i:=0;
    while i<=TagList.Count-1 do
      if TagList[i]='' then TagList.Delete(i)
      else inc(i);
    end;

begin
  if OpenDialog1.Execute then begin
    IndexFileName:=OpenDialog1.FileName;
    Caption:='Help Setting to '+IndexFileName;
    HrefList:=TStringList.Create;
    TagList:=TStringList.Create;
    try with HrefList do begin
      if not FileExists(ExtractFilePath(Application.ExeName)+'Help_lst.txt') then begin
        MessageDlg('File "Help_lst.txt" cannot be found.', mtError, [mbOk], 0);
        exit;
        end;
      TagList.LoadFromFile(ExtractFilePath(Application.ExeName)+'Help_lst.txt');
      TagList.Text:=UpperCase(TagList.Text);
      DelTagSpaces;
      TagCount:=TagList.Count;
      LoadFromFile(IndexFileName);
      FoundAt := -1;
      Str := UpperCase(Text);
      clear;
      BegSize:=Length(Str);
      ProgressBar1.Position:=0;
      FindMemo.Enabled:=false;
      NoFindMemo.Enabled:=false;
      OpenButton.Enabled:=false;
      SaveButton.Enabled:=false;
      Refresh;
      while FoundAt <> 0 do begin
        FoundAt := Pos('<A',Str);
        if FoundAt <> 0 then begin
          GtStr := Copy(Str,FoundAt,Pos('</A',Str)-FoundAt);
          AtxtAt:=Pos('>',GtStr)+1;
          AhrefAt:=Pos('HREF',Copy(GtStr,1,AtxtAt-1));
          if AhrefAt<>0 then begin
            Ahref:=GetHref;
            if (not WoHTTPCheckBox.Checked)or(Pos('HTTP:',Ahref)=0) then begin
              Atxt := Copy(GtStr,AtxtAt,length(GtStr)-AtxtAt+1);
              ATag:=FindTag;
              if ATag<>'' then begin
                Add(ATag+'='+AHref);
                FindLabel.Caption:=IntToStr(TagCount-TagList.Count)+'/'+
                  IntToStr(TagCount);
                FindLabel.Refresh;
                end;
              end;
            end;
          System.Delete(Str,1,FoundAt+Length(GtStr));
          ProgressBar1.Position:=100-Length(Str)*100 div BegSize;
          Application.ProcessMessages;
          if FlExit then break;
          end;
        end;
      ProgressBar1.Position:=0;
      FindMemo.Lines.Assign(HrefList);
      NoFindMemo.Lines.Assign(TagList);
      FindMemo.Enabled:=true;
      NoFindMemo.Enabled:=true;
      SaveButton.Enabled:=true;
      OpenButton.Enabled:=true;
      end;
    finally
      HrefList.Free;
      TagList.Free;
      FlExit:=false;
      end;
    end;
  end;

procedure TSetHelpForm.FormShow(Sender: TObject);
begin
  Caption:='Help Setting';
  SaveButton.Enabled:=false;
  FindMemo.Text:='Founded tags';
  NoFindMemo.Text:='Not founded tags';
  FindMemo.Enabled:=false;
  NoFindMemo.Enabled:=false;
  ProgressBar1.Position:=0;
  FindLabel.Caption:='0/0';
  end;

procedure TSetHelpForm.SaveButtonClick(Sender: TObject);
var
  NewString: string;
  //ClickedOK: Boolean;
  IniFileList:TStringList;
  IniFileName:string;
  //FExit:boolean;

  procedure DelFindMemoSpaces;
  var i:integer;
  begin
    i:=0;
    while i<=FindMemo.Lines.Count-1 do
      if FindMemo.Lines[i]='' then FindMemo.Lines.Delete(i)
      else inc(i);
    end;

begin
  IniFileList:=TStringList.Create;
  try
    NewString:='OK'; //ExtractFileName(IndexFileName);
    IniFileName:=ExtractFilePath(Application.ExeName)+'help.ini';

    {if FileExists(IniFileName) then
      IniFileList.LoadFromFile(IniFileName);
    repeat
      FExit:=true;
      ClickedOK:=InputQuery(' ',
        '   :', NewString);
      if not ClickedOK then exit;
      if IniFileList.IndexOfName('File'+NewString)<>-1 then
        if MessageDlg(NewString+#13+
          '   .'+#13+#13+' ?',
          mtWarning, [mbYes,mbNo], 0) = mrYes then begin
          ShowMessage('Sorry, It Dosn''t work yet :(');
          FExit:=false;
        end else FExit:=false;
      until FExit;} //     :(

    IniFileList.Insert(0,'File'+NewString+'='+IndexFileName);
    IniFileList.Append(#13+#10+'['+NewString+']');
    DelFindMemoSpaces;
    IniFileList.AddStrings(FindMemo.Lines);
    IniFileList.SaveToFile(IniFileName);
  finally
    IniFileList.Free;
    end;
  Close;
  end;

procedure TSetHelpForm.CancelButtonClick(Sender: TObject);
begin
  FlExit:=true;
  if not OpenButton.Enabled then exit;
  Close;
  end;

end.
