{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit Options;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Grids, menus;

type
  PShortCutItem=^TShortCutItem;
  TShortCutItem=record
    MenuItem:TMenuItem;
    Caption:string;
    Shortcut:TShortcut;
    end;
  TOptForm = class(TForm)
    OkButton: TButton;
    CancelButton: TButton;
    OptPageControl: TPageControl;
    GeneralTabSheet: TTabSheet;
    ARDelayLabel: TLabel;
    Bevel1: TBevel;
    HTMLDefaultExtLabel: TLabel;
    ARDelayEdit: TEdit;
    DelBakFilesCheckBox: TCheckBox;
    HTMLDefaultExtEdit: TEdit;
    ShowLogoCheckBox: TCheckBox;
    BrowsersTabSheet: TTabSheet;
    B1BrNameEdit: TEdit;
    B1PathToBrEdit: TEdit;
    B1NumLabel: TLabel;
    B1OpenButton: TButton;
    BAutoSearchButton: TButton;
    B2BrNameEdit: TEdit;
    B2PathToBrEdit: TEdit;
    B2NumLabel: TLabel;
    B2OpenButton: TButton;
    B3BrNameEdit: TEdit;
    B3PathToBrEdit: TEdit;
    B3NumLabel: TLabel;
    B3OpenButton: TButton;
    B4BrNameEdit: TEdit;
    B4PathToBrEdit: TEdit;
    B4NumLabel: TLabel;
    B4OpenButton: TButton;
    BBrNameLabel: TLabel;
    BPathToBrLabel: TLabel;
    OptOpenDialog: TOpenDialog;
    ShortcutsTabSheet: TTabSheet;
    SStringGrid: TStringGrid;
    SHotKey: THotKey;
    SKeyShortcutLabel: TLabel;
    SApplyButton: TButton;
    SAutoCheckBox: TCheckBox;
    SHotKeyEdit: TEdit;
    SimpleImgToBrCheckBox: TCheckBox;
    TitleTabSheet: TTabSheet;
    TPNameGroupBox: TGroupBox;
    TPNFirstRadioButton: TRadioButton;
    TPNLastRadioButton: TRadioButton;
    TPNVisibleCheckBox: TCheckBox;
    TPNIncludeVersionCheckBox: TCheckBox;
    TPNBevel: TBevel;
    TBFGroupBox: TGroupBox;
    TBFBevel: TBevel;
    TBFTitleRadioButton: TRadioButton;
    TBFLocationRadioButton: TRadioButton;
    TBFVisibleCheckBox: TCheckBox;
    TBFFullPathRadioButton: TRadioButton;
    TBFRelativeRadioButton: TRadioButton;
    TBFNameRadioButton: TRadioButton;
    TEFGroupBox: TGroupBox;
    TEFBevel: TBevel;
    TEFVisibleCheckBox: TCheckBox;
    TEFFullPathRadioButton: TRadioButton;
    TEFRelativeRadioButton: TRadioButton;
    TEFNameRadioButton: TRadioButton;
    TFFGroupBox: TGroupBox;
    TFFBrowserRadioButton: TRadioButton;
    TFFEditorRadioButton: TRadioButton;
    Allow1copyCheckBox: TCheckBox;
    ScrollBoxGroupBox: TGroupBox;
    ScrollBoxHeightLabel: TLabel;
    ScrollBoxHeightEdit: TEdit;
    ScrollBoxWidthLabel: TLabel;
    ScrollBoxWidthEdit: TEdit;
    ScrollBox640x480Button: TButton;
    ScrollBox800x600Button: TButton;
    ScrollBox1024x768Button: TButton;
    IntegrationTabSheet: TTabSheet;
    ContextMenuCheckBox: TCheckBox;
    IEGroupBox: TGroupBox;
    ViewSourceCheckBox: TCheckBox;
    EditWithCheckBox: TCheckBox;
    StartMenuCheckBox: TCheckBox;
    DesktopCheckBox: TCheckBox;
    SendMenuCheckBox: TCheckBox;
    DoNotBrNavCheckBox: TCheckBox;
    ProgramMenuCheckBox: TCheckBox;
    DefEditorCheckBox: TCheckBox;
    Bevel2: TBevel;
    DefParserRadioGroup: TRadioGroup;
    procedure OkButtonClick(Sender: TObject);
    procedure ScrollBoxHeightEditExit(Sender: TObject);
    procedure ARDelayEditExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure BOpenButtonClick(Sender: TObject);
    procedure BAutoSearchButtonClick(Sender: TObject);
    procedure BBrNameEditChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SStringGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure SApplyButtonClick(Sender: TObject);
    procedure SStringGridEnter(Sender: TObject);
    procedure OptPageControlChange(Sender: TObject);
    procedure SAutoCheckBoxClick(Sender: TObject);
    procedure TPNVisibleCheckBoxClick(Sender: TObject);
    procedure TBFVisibleCheckBoxClick(Sender: TObject);
    procedure TEFVisibleCheckBoxClick(Sender: TObject);
    procedure ScrollBoxWidthEditExit(Sender: TObject);
    procedure ScrollBoxSizeButtonClick(Sender: TObject);
  private
    { Private declarations }
    SContextMenu:boolean;
    SDefEditor:boolean;
    SEditWith:boolean;
    SViewSource:boolean;
    SProgramMenu:boolean;
    SStartMenu:boolean;
    SDesktop:boolean;
    SSendMenu:boolean;
    SScrollBoxWidth,SScrollBoxHeight:integer;
    SARDelay:integer;
    InputIntError:boolean;
    procedure AllEditExit;
    procedure EditTextToNumInt(AEdit:TEdit;var ANum:integer);
  public
    { Public declarations }
    SShortcutsList:TList;
    procedure GetShortcuts;
    procedure SetShortcuts;
  end;

var
  FShowLogo:boolean;
  OptForm: TOptForm;

implementation

uses Main, FileWork, Files, RegWork;

{$R *.DFM}

procedure TOptForm.EditTextToNumInt(AEdit:TEdit;var ANum:integer);
//String AEdit.Text into number ANum
var
  XCode:integer;
begin
  val(AEdit.Text,ANum,XCode);
  if XCode>0 then begin
    InputIntError:=true;
    MessageDlg('Error input integer number.',mtError,[mbOK],0);
    ActiveControl:=AEdit;
    AEdit.SelStart:=XCode-1;
    AEdit.SelLength:=1;
    end;
  InputIntError:=false;
  end;

procedure TOptForm.AllEditExit;
begin
  if (ActiveControl is TEdit)and(Assigned(TEdit(ActiveControl).onExit)) then
    TEdit(ActiveControl).OnExit(self);
  end;

procedure TOptForm.OkButtonClick(Sender: TObject);
begin
  AllEditExit;
  if InputIntError then exit;
  //--- General ---
  MainForm.ScrollBoxWidth:=SScrollBoxWidth;
  MainForm.ScrollBoxHeight:=SScrollBoxHeight;
  MainForm.ARDelay:=SARDelay;
  FileWork.HTMLDefaultExt:=HTMLDefaultExtEdit.Text;
  Files.DefParser:=DefParserRadioGroup.ItemIndex+1;
  FileWork.DelBakFiles:=DelBakFilesCheckBox.Checked;
  Options.FShowLogo:=ShowLogoCheckBox.Checked;
  MainForm.SimpleImgToBr:=SimpleImgToBrCheckBox.Checked;
  MainForm.DoNotBrNav:=DoNotBrNavCheckBox.Checked;
  RegWork.SetAllow1copy(Allow1copyCheckBox.Checked);
  MainForm.SetWBrSize;
  //--- Title ---
  with MainForm.TitleFormat do begin
    PNVisible:=TPNVisibleCheckBox.Checked;
    BFVisible:=TBFVisibleCheckBox.Checked;
    EFVisible:=TEFVisibleCheckBox.Checked;
    if TBFTitleRadioButton.Checked then BFType:=bfTitle
    else if TBFLocationRadioButton.Checked then BFType:=bfLocation
    else if TBFFullPathRadioButton.Checked then BFType:=bfFullPath
    else if TBFRelativeRadioButton.Checked then BFType:=bfRelative
    else if TBFNameRadioButton.Checked then BFType:=bfName;
    if TEFFullPathRadioButton.Checked then EFType:=efFullPath
    else if TEFRelativeRadioButton.Checked then EFType:=efRelative
    else if TEFNameRadioButton.Checked then EFType:=efName;
    PNIncludeVersion:=TPNIncludeVersionCheckBox.Checked;
    PNFirst:=TPNFirstRadioButton.Checked;
    FFBrowser:=TFFBrowserRadioButton.Checked;
    end;
  MainForm.SetMainFormCaption;
  //--- Shortcuts ---
  SetShortcuts;
  //--- Browsers ---
  with MainForm do begin
    B1BrName:=B1BrNameEdit.Text; B1PathToBr:=B1PathToBrEdit.Text;
    B2BrName:=B2BrNameEdit.Text; B2PathToBr:=B2PathToBrEdit.Text;
    B3BrName:=B3BrNameEdit.Text; B3PathToBr:=B3PathToBrEdit.Text;
    B4BrName:=B4BrNameEdit.Text; B4PathToBr:=B4PathToBrEdit.Text;
    end;
  //--- Integration ---
  if SContextMenu<>ContextMenuCheckBox.Checked then
    if ContextMenuCheckBox.Checked
      then RegWork.SetContextMenu(Application.ExeName)
      else RegWork.DelContextMenu;
  if SEditWith<>EditWithCheckBox.Checked then
    if EditWithCheckBox.Checked
      then RegWork.SetEditWith(Application.ExeName)
      else RegWork.DelEditWith;
  if SDefEditor<>DefEditorCheckBox.Checked then
    if DefEditorCheckBox.Checked
      then RegWork.SetDefEditor(Application.ExeName)
      else RegWork.DelDefEditor;
  if SViewSource<>ViewSourceCheckBox.Checked then
    if ViewSourceCheckBox.Checked
      then RegWork.SetViewSource(Application.ExeName)
      else RegWork.DelViewSource;
  if SProgramMenu<>ProgramMenuCheckBox.Checked then
    if ProgramMenuCheckBox.Checked
      then RegWork.SetProgramMenu(Application.ExeName)
      else RegWork.DelProgramMenu;
  if SStartMenu<>StartMenuCheckBox.Checked then
    if StartMenuCheckBox.Checked
      then RegWork.SetExplorer('Start Menu',Application.ExeName)
      else RegWork.DelExplorer('Start Menu');
  if SDesktop<>DesktopCheckBox.Checked then
    if DesktopCheckBox.Checked
      then RegWork.SetExplorer('Desktop',Application.ExeName)
      else RegWork.DelExplorer('Desktop');
  if SSendMenu<>SendMenuCheckBox.Checked then
    if SendMenuCheckBox.Checked
      then RegWork.SetExplorer('SendTo',Application.ExeName)
      else RegWork.DelExplorer('SendTo');
  Close;
  end;

procedure TOptForm.ScrollBoxWidthEditExit(Sender: TObject);
begin
  EditTextToNumInt(ScrollBoxWidthEdit,SScrollBoxWidth);
  end;

procedure TOptForm.ScrollBoxHeightEditExit(Sender: TObject);
begin
  EditTextToNumInt(ScrollBoxHeightEdit,SScrollBoxHeight);
  end;

procedure TOptForm.ARDelayEditExit(Sender: TObject);
begin
  EditTextToNumInt(ARDelayEdit,SARDelay);
  end;

procedure TOptForm.FormShow(Sender: TObject);
begin
  //--- General ---
  SScrollBoxWidth:=MainForm.ScrollBoxWidth;
  ScrollBoxWidthEdit.Text:=IntToStr(SScrollBoxWidth);
  SScrollBoxHeight:=MainForm.ScrollBoxHeight;
  ScrollBoxHeightEdit.Text:=IntToStr(SScrollBoxHeight);
  SARDelay:=MainForm.ARDelay;
  ARDelayEdit.Text:=IntToStr(SARDelay);
  HTMLDefaultExtEdit.Text:=FileWork.HTMLDefaultExt;
  DefParserRadioGroup.ItemIndex:=Files.DefParser-1;
  DelBakFilesCheckBox.Checked:=DelBakFiles;
  ShowLogoCheckBox.Checked:=Options.FShowLogo;
  SimpleImgToBrCheckBox.Checked:=MainForm.SimpleImgToBr;
  DoNotBrNavCheckBox.Checked:=MainForm.DoNotBrNav;
  Allow1copyCheckBox.Checked:=RegWork.CmpAllow1copy;
  //--- Title ---
  with MainForm.TitleFormat do begin
    TPNVisibleCheckBox.Checked:=PNVisible;
    TBFVisibleCheckBox.Checked:=BFVisible;
    TEFVisibleCheckBox.Checked:=EFVisible;
    case BFType of
      bfTitle    : TBFTitleRadioButton.Checked:=true;
      bfLocation : TBFLocationRadioButton.Checked:=true;
      bfFullPath : TBFFullPathRadioButton.Checked:=true;
      bfRelative : TBFRelativeRadioButton.Checked:=true;
      bfName     : TBFNameRadioButton.Checked:=true;
      end;
    case EFType of
      efFullPath : TEFFullPathRadioButton.Checked:=true;
      efRelative : TEFRelativeRadioButton.Checked:=true;
      efName     : TEFNameRadioButton.Checked:=true;
      end;
    TPNIncludeVersionCheckBox.Checked:=PNIncludeVersion;
    TPNFirstRadioButton.Checked:=PNFirst;
    TPNLastRadioButton.Checked:=not PNFirst;
    TFFBrowserRadioButton.Checked:=FFBrowser;
    TFFEditorRadioButton.Checked:=not FFBrowser;
    end;
  //--- Shortcuts ---
  GetShortcuts;
  //--- Browsers ---
  with MainForm do begin
    B1BrNameEdit.Text:=B1BrName; B1PathToBrEdit.Text:=B1PathToBr;
    B2BrNameEdit.Text:=B2BrName; B2PathToBrEdit.Text:=B2PathToBr;
    B3BrNameEdit.Text:=B3BrName; B3PathToBrEdit.Text:=B3PathToBr;
    B4BrNameEdit.Text:=B4BrName; B4PathToBrEdit.Text:=B4PathToBr;
    end;
  //--- Integration ---
  SContextMenu:=RegWork.CheckContextMenu;
  ContextMenuCheckBox.Checked:=SContextMenu;
  SDefEditor:=RegWork.CheckDefEditor(Application.ExeName);
  DefEditorCheckBox.Checked:=SDefEditor;
  SEditWith:=RegWork.CheckEditWith;
  EditWithCheckBox.Checked:=SEditWith;
  SViewSource:=RegWork.CheckViewSource(Application.ExeName);
  ViewSourceCheckBox.Checked:=SViewSource;
  SProgramMenu:=RegWork.CheckProgramMenu;
  ProgramMenuCheckBox.Checked:=SProgramMenu;
  SStartMenu:=RegWork.CheckExplorer('Start Menu');
  StartMenuCheckBox.Checked:=SStartMenu;
  SDesktop:=RegWork.CheckExplorer('Desktop');
  DesktopCheckBox.Checked:=SDesktop;
  SSendMenu:=RegWork.CheckExplorer('SendTo');
  SendMenuCheckBox.Checked:=SSendMenu;
  end;

procedure TOptForm.CancelButtonClick(Sender: TObject);
begin
  Close;
  end;

procedure TOptForm.BOpenButtonClick(Sender: TObject);
var
  XPathToBr:string;
begin
  XPathToBr:='';
  if Sender=B1OpenButton then XPathToBr:=B1PathToBrEdit.Text;
  if Sender=B2OpenButton then XPathToBr:=B2PathToBrEdit.Text;
  if Sender=B3OpenButton then XPathToBr:=B3PathToBrEdit.Text;
  if Sender=B4OpenButton then XPathToBr:=B4PathToBrEdit.Text;
  OptOpenDialog.InitialDir:=ExtractFileDir(XPathToBr);
  OptOpenDialog.FileName:=XPathToBr;
  if OptOpenDialog.Execute then begin
    XPathToBr:=OptOpenDialog.FileName;
    if Sender=B1OpenButton then B1PathToBrEdit.Text:=XPathToBr;
    if Sender=B2OpenButton then B2PathToBrEdit.Text:=XPathToBr;
    if Sender=B3OpenButton then B3PathToBrEdit.Text:=XPathToBr;
    if Sender=B4OpenButton then B4PathToBrEdit.Text:=XPathToBr;
    end;
  end;

procedure TOptForm.BAutoSearchButtonClick(Sender: TObject);
begin
  RegWork.BrowsersAutoSearch;
  end;

procedure TOptForm.BBrNameEditChange(Sender: TObject);
begin
  BAutoSearchButton.Enabled:=(
    (B1BrNameEdit.Text='')or
    (B2BrNameEdit.Text='')or
    (B3BrNameEdit.Text='')or
    (B4BrNameEdit.Text='')
    );
  end;

function ListSortCmp(AItem1,AItem2:Pointer):integer;
begin
  result:=CompareStr(PShortcutItem(AItem1)^.Caption,
                     PShortcutItem(AItem2)^.Caption)
  end;

procedure TOptForm.FormCreate(Sender: TObject);
begin
  SStringGrid.Cells[0,0]:='Command';
  SStringGrid.Cells[1,0]:='Key Shortcut';
  end;

procedure TOptForm.GetShortcuts;
var
  i:integer;

  procedure AddShortcut(AMenuItem:TMenuItem);
  var
    XShortCutItem:PShortCutItem;
  begin
    New(XShortCutItem);
    XShortCutItem.MenuItem := AMenuItem;
    XShortCutItem.Caption  := StripHotKey(AMenuItem.Caption);
    XShortCutItem.Shortcut := AMenuItem.Shortcut;
    SShortcutsList.Add(XShortCutItem);
    end;

  procedure RecurMenu(AMenuItem:TMenuItem);
  var
    i:integer;
  begin
    if AMenuItem.Tag=1 then AddShortcut(AMenuItem);
    i:=0;
    while i<AMenuItem.Count do begin
      RecurMenu(AMenuItem.Items[i]);
      inc(i);
      end;
    end;

begin
  MainForm.FPreviewinBrowserClick(self); //Setting Captions
  if Assigned(SShortcutsList) then begin
    for i:=0 to SShortcutsList.Count-1 do Dispose(PShortcutItem(SShortcutsList.Items[i]));
    SShortcutsList.Free;
    end;
  SShortcutsList:=TList.Create;
  RecurMenu(MainForm.MainMenu.Items);
  SShortcutsList.Sort(ListSortCmp);
  SStringGrid.RowCount:=SShortcutsList.Count+1;
  for i:=0 to SShortcutsList.Count-1 do begin
    SStringGrid.Cells[0,i+1] := PShortcutItem(SShortcutsList.Items[i])^.Caption;
    SStringGrid.Cells[1,i+1] :=
      ShortcutToText(PShortcutItem(SShortcutsList.Items[i])^.Shortcut);
    end;
  end;

procedure TOptForm.SetShortcuts;
var
  i:integer;
begin
  for i:=0 to SShortcutsList.Count-1 do
    PShortcutItem(SShortcutsList.Items[i])^.MenuItem.Shortcut:=
      PShortcutItem(SShortcutsList.Items[i])^.Shortcut;
  end;

procedure TOptForm.SStringGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var XShortcut:TShortcut;
begin
  XShortcut:=PShortcutItem(SShortcutsList.Items[ARow-1])^.Shortcut;
  if SAutoCheckBox.Checked
    then SHotKey.HotKey:=XShortcut
    else SHotKeyEdit.Text:=ShortcutToText(XShortcut);
  CanSelect:=true;
  end;

procedure TOptForm.SApplyButtonClick(Sender: TObject);
var
  XShortcut:TShortcut;
  XStr:string;
  XInt:integer;
begin
  if SAutoCheckBox.Checked
    then XShortcut:=SHotKey.HotKey
    else XShortcut:=TextToShortcut(SHotKeyEdit.Text);
  XStr:=ShortcutToText(XShortcut);
  if XStr<>'' then begin
    XInt:=SStringGrid.Cols[1].IndexOf(XStr);
    if (XInt<>-1) and (XInt<>SStringGrid.Row) and
      (MessageDlg('Key Shortcut "'+XStr+'" already exists.'+#13+
        '(Command "'+SStringGrid.Cols[0].Strings[SStringGrid.Cols[1].IndexOf(XStr)]+'")'+#13+#13+
        'Set Key Shortcut?',
        mtWarning, [mbYes,mbNo], 0) <> idYes) then exit;
    end;
  PShortcutItem(SShortcutsList.Items[SStringGrid.Row-1])^.Shortcut:=XShortcut;
  SStringGrid.Cells[1,SStringGrid.Row]:=XStr;
  end;

procedure TOptForm.SStringGridEnter(Sender: TObject);
var XBool:boolean;
begin
  SStringGridSelectCell(self,SStringGrid.Col,SStringGrid.Row,XBool);
  end;

procedure TOptForm.OptPageControlChange(Sender: TObject);
begin
  if OptPageControl.ActivePage=ShortcutsTabSheet then SStringGridEnter(self);
  end;

procedure TOptForm.SAutoCheckBoxClick(Sender: TObject);
begin
  SHotKeyEdit.Enabled := not SAutoCheckBox.Checked;
  SHotKeyEdit.Visible := not SAutoCheckBox.Checked;
  SHotKey.Enabled     :=     SAutoCheckBox.Checked;
  SHotKey.Visible     :=     SAutoCheckBox.Checked;
  if SAutoCheckBox.Checked then begin
    SHotKey.HotKey:=TextToShortcut(SHotKeyEdit.Text);
    SKeyShortcutLabel.FocusControl:=SHotKey;
  end else begin
    SHotKeyEdit.Text:=ShortcutToText(SHotKey.HotKey);
    SKeyShortcutLabel.FocusControl:=SHotKeyEdit;
    end;
  end;

procedure TOptForm.TPNVisibleCheckBoxClick(Sender: TObject);
var checked:boolean;
begin
  checked:=TBFVisibleCheckBox.Checked or TEFVisibleCheckBox.Checked;
  if not checked then TPNVisibleCheckBox.Checked:=true;
  TPNIncludeVersionCheckBox.Enabled:=TPNVisibleCheckBox.Checked;
  TPNFirstRadioButton.Enabled:=checked and TPNVisibleCheckBox.Checked;
  TPNLastRadioButton.Enabled:=checked and TPNVisibleCheckBox.Checked;
  end;

procedure TOptForm.TBFVisibleCheckBoxClick(Sender: TObject);
var checked:boolean;
begin
  checked:=TBFVisibleCheckBox.Checked;
  TBFTitleRadioButton.Enabled:=checked;
  TBFLocationRadioButton.Enabled:=checked;
  TBFFullPathRadioButton.Enabled:=checked;
  TBFRelativeRadioButton.Enabled:=checked;
  TBFNameRadioButton.Enabled:=checked;
  checked:=checked and TEFVisibleCheckBox.Checked;
  TFFBrowserRadioButton.Enabled:=checked;
  TFFEditorRadioButton.Enabled:=checked;
  TPNVisibleCheckBoxClick(self);
  end;

procedure TOptForm.TEFVisibleCheckBoxClick(Sender: TObject);
var checked:boolean;
begin
  checked:=TEFVisibleCheckBox.Checked;
  TEFFullPathRadioButton.Enabled:=checked;
  TEFRelativeRadioButton.Enabled:=checked;
  TEFNameRadioButton.Enabled:=checked;
  checked:=checked and TBFVisibleCheckBox.Checked;
  TFFBrowserRadioButton.Enabled:=checked;
  TFFEditorRadioButton.Enabled:=checked;
  TPNVisibleCheckBoxClick(self);
  end;

procedure TOptForm.ScrollBoxSizeButtonClick(Sender: TObject);
var XWidth,XHeight:integer;
begin
  XWidth:=0; XHeight:=0;
  if Sender=ScrollBox640x480Button then begin XWidth:=640; XHeight:=480 end;
  if Sender=ScrollBox800x600Button then begin XWidth:=800; XHeight:=600 end;
  if Sender=ScrollBox1024x768Button then begin XWidth:=1024; XHeight:=768 end;
  SScrollBoxWidth:=XWidth;
  SScrollBoxHeight:=XHeight;
  ScrollBoxWidthEdit.Text:=IntToStr(XWidth);
  ScrollBoxHeightEdit.Text:=IntToStr(XHeight);
  end;

end.
