{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit RegWork;

interface

uses
  Windows, Registry, Classes, Graphics, Menus, SysUtils,
  Main;

const
  AppRegistryKey = '\Software\PROFOX\CoolEdit';

procedure RegistryLoad;
procedure RegistrySave;
procedure BrowsersAutoSearch;
function RWGetActiveFile(var AFileData:TFileData):boolean;
procedure RWSetActiveFile(AFileData:TFileData);
function CmpShowLogo:boolean;
function CmpAllow1copy:boolean;
procedure SetAllow1copy(AValue:boolean);
procedure RWSetParamStr1;
function RWGetParamStr1:string;
function CheckContextMenu:boolean;
procedure SetContextMenu(AExeName:string);
procedure DelContextMenu;
function CheckEditWith:boolean;
procedure SetEditWith(AExeName:string);
procedure DelEditWith;
function CheckDefEditor(AExeName:string):boolean;
procedure SetDefEditor(AExeName:string);
procedure DelDefEditor;
function CheckViewSource(AExeName:string):boolean;
procedure SetViewSource(AExeName:string);
procedure DelViewSource;
function CheckProgramMenu:boolean;
procedure SetProgramMenu(AExeName:string);
procedure DelProgramMenu;
function CheckExplorer(AFolder:string):boolean;
procedure SetExplorer(AFolder:string;AExeName:string);
procedure DelExplorer(AFolder:string);
procedure SetURLHistoryBox;

implementation

uses
  FileWork, Files, Options, OpenWeb,
  FileCtrl,
  ShlObj, ActiveX, ComObj;

procedure RegistryLoad;
var
  Reg: TRegistry;
  XBool:boolean;
  XInt:integer;
  XStringList:TStringList;
  i:integer;
  {--}
  function RegReadInteger(AName:string; var AValue:integer):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadInteger(AName);
      result:=true;
      end;
    end;
  function RegReadBool(AName:string; var AValue:boolean):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadBool(AName);
      result:=true;
      end;
    end;
  function RegReadString(AName:string; var AValue:string):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadString(AName);
      result:=true;
      end;
    end;
  procedure RegSetShortcut(AMenuItemName:string;AShortcut:TShortcut);
  var XMenuItem:TMenuItem;
    {--}
    procedure FindMenuItem(AMenuItem:TMenuItem);
    var i:integer;
    begin
      if XMenuItem<>nil then exit;
      if AMenuItem.Name=AMenuItemName then begin
        XMenuItem:=AMenuItem; exit;
        end;
      for i:=0 to AMenuItem.Count-1 do FindMenuItem(AMenuItem.Items[i]);
      end;
    {--}
  begin
    XMenuItem:=nil;
    FindMenuItem(MainForm.MainMenu.Items);
    if XMenuItem<>nil then XMenuItem.Shortcut:=AShortcut;
    end;
  {--}
begin
  with MainForm.HTMLParser do begin
    RegistryKey:=AppRegistryKey+'\Editor';
    StylesFromRegistry(true,'');
    end;
  with MainForm.PlainTextParser do begin
    RegistryKey:=AppRegistryKey+'\Editor';
    StylesFromRegistry(true,'');
    end;
  with MainForm.SQLParser do begin
    RegistryKey:=AppRegistryKey+'\Editor';
    StylesFromRegistry(true,'');
    end;
  Reg := TRegistry.Create;
  with MainForm do try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(AppRegistryKey) then begin
      if RegReadInteger('EditorState',XInt) then EditorState:=TEditorState(XInt);
      RegReadInteger('ARDelay',ARDelay);
      RegReadInteger('ScrollBoxWidth',ScrollBoxWidth);
      RegReadInteger('ScrollBoxHeight',ScrollBoxHeight);
      RegReadInteger('DefParser',Files.DefParser);
      RegReadBool('DelBakFiles',FileWork.DelBakFiles);
      RegReadBool('ShowLogo',Options.FShowLogo);
      RegReadBool('SimpleImgToBr',SimpleImgToBr);
      RegReadBool('DoNotBrNav',DoNotBrNav);
      with TitleFormat do begin
        RegReadBool('PNVisible',PNVisible);
        RegReadBool('BFVisible',BFVisible);
        RegReadBool('EFVisible',EFVisible);
        if RegReadInteger('BFType',XInt) then BFType:=TBFType(XInt);
        if RegReadInteger('EFType',XInt) then EFType:=TEFType(XInt);
        RegReadBool('PNIncludeVersion',PNIncludeVersion);
        RegReadBool('PNFirst',PNFirst);
        RegReadBool('FFBrowser',FFBrowser);
        end;
      if RegReadBool('WorkOffline',XBool) then WebBrowser.Offline:=XBool;
      if RegReadBool('VScrollBox',XBool) then begin
        VScrollBox.Checked:=not XBool;
        VScrollBoxClick(nil);
        end;
      if RegReadBool('VStatusBar',XBool) then begin
        VStatusBar.Checked:=not XBool;
        VStatusBarClick(nil); //VStatusBarClick changes SplitterPercent
        end;        
      RegReadInteger('SplitterPercent',SplitterPercent);
      if RegReadBool('GroupedUndo',XBool) then begin
        OGroupedUndo.Checked:=not XBool;
        OGroupedUndoClick(nil);
        end;
      if RegReadBool('UndoAfterSave',XBool) then begin
        OUndoAfterSave.Checked:=not XBool;
        OUndoAfterSaveClick(nil);
        end;
      if RegReadBool('AutoRefresh',XBool) then begin
        OAutoRefresh.Checked:=not XBool;
        OAutoRefreshClick(nil);
        end;
      RegReadString('HTMLDefaultExt',FileWork.HTMLDefaultExt);
      RegReadString('B1BrName',B1BrName); RegReadString('B1PathToBr',B1PathToBr);
      RegReadString('B2BrName',B2BrName); RegReadString('B2PathToBr',B2PathToBr);
      RegReadString('B3BrName',B3BrName); RegReadString('B3PathToBr',B3PathToBr);
      RegReadString('B4BrName',B4BrName); RegReadString('B4PathToBr',B4PathToBr);
      if Reg.OpenKeyReadOnly(AppRegistryKey+'\ReOpen') then begin
        Reg.GetValueNames(ReOpenList);
        ReOpenListCrop;
        end;
      if Reg.OpenKeyReadOnly(AppRegistryKey+'\Shortcuts') then begin
        XStringList:=TStringList.Create;
        try
          Reg.GetValueNames(XStringList);
          for i:=0 to XStringList.Count-1 do
            if RegReadInteger(XStringList.Strings[i],XInt) then
              RegSetShortcut(XStringList.Strings[i],TShortcut(XInt));
        finally
          XStringList.Free;
          end;
        end;
      end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure RegistrySave;
var
  Reg: TRegistry;
  i:integer;
begin
  Reg := TRegistry.Create;
  with MainForm do try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(AppRegistryKey,True)
    then with Reg do begin
      WriteInteger('EditorState',ord(EditorState));
      WriteInteger('SplitterPercent',SplitterPercent);
      WriteInteger('ARDelay',ARDelay);
      WriteInteger('ScrollBoxWidth',ScrollBoxWidth);
      WriteInteger('ScrollBoxHeight',ScrollBoxHeight);
      WriteInteger('DefParser',Files.DefParser);
      WriteBool('DelBakFiles',DelBakFiles);
      WriteBool('ShowLogo',Options.FShowLogo);
      WriteBool('SimpleImgToBr',SimpleImgToBr);
      WriteBool('DoNotBrNav',DoNotBrNav);
      WriteBool('VScrollBox',VScrollBox.Checked);
      WriteBool('VStatusBar',VStatusBar.Checked);
      WriteBool('GroupedUndo',OGroupedUndo.Checked);
      WriteBool('UndoAfterSave',OUndoAfterSave.Checked);
      WriteBool('AutoRefresh',OAutoRefresh.Checked);
      WriteBool('WorkOffline',WebBrowser.Offline);
      with TitleFormat do begin
        WriteBool('PNVisible',PNVisible);
        WriteBool('BFVisible',BFVisible);
        WriteBool('EFVisible',EFVisible);
        WriteInteger('BFType',integer(BFType));
        WriteInteger('EFType',integer(EFType));
        WriteBool('PNIncludeVersion',PNIncludeVersion);
        WriteBool('PNFirst',PNFirst);
        WriteBool('FFBrowser',FFBrowser);
        end;
      WriteString('HTMLDefaultExt',FileWork.HTMLDefaultExt);
      WriteString('B1BrName',B1BrName); WriteString('B1PathToBr',B1PathToBr);
      WriteString('B2BrName',B2BrName); WriteString('B2PathToBr',B2PathToBr);
      WriteString('B3BrName',B3BrName); WriteString('B3PathToBr',B3PathToBr);
      WriteString('B4BrName',B4BrName); WriteString('B4PathToBr',B4PathToBr);
      WriteString('URLHistoryBox',OpenWebForm.URLHistoryBox.Items.Text);
      Reg.DeleteKey(AppRegistryKey+'\ReOpen');
      if Reg.OpenKey(AppRegistryKey+'\ReOpen',True) then
        for i:=0 to ReOpenList.Count-1 do
          WriteString(ReOpenList.Strings[i],'');
      if Reg.OpenKey(AppRegistryKey+'\Shortcuts',True) then
        with OptForm do begin
          GetShortcuts;
          for i:=0 to SShortcutsList.Count-1 do
            WriteInteger(PShortcutItem(SShortcutsList.Items[i])^.MenuItem.Name,
                         PShortcutItem(SShortcutsList.Items[i])^.Shortcut);
          end;
      end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure BrowsersAutoSearch;
var
  Reg: TRegistry;
  XBrName,XPathToBr:string;

  procedure SetPathToBr(ABrName,APathToBr:string);
  begin
    if OptForm.BAutoSearchButton.Enabled then
      with OptForm do begin
        if B1BrNameEdit.Text='' then begin
          B1BrNameEdit.Text:=ABrName; B1PathToBrEdit.Text:=APathToBr; exit;
          end;
        if B2BrNameEdit.Text='' then begin
          B2BrNameEdit.Text:=ABrName; B2PathToBrEdit.Text:=APathToBr; exit;
          end;
        if B3BrNameEdit.Text='' then begin
          B3BrNameEdit.Text:=ABrName; B3PathToBrEdit.Text:=APathToBr; exit;
          end;
        if B4BrNameEdit.Text='' then begin
          B4BrNameEdit.Text:=ABrName; B4PathToBrEdit.Text:=APathToBr; exit;
          end;
        end;
    end;

  function GetPathToBr(AExeName:string;out APathToBr:string):boolean;
  const
    AppPaths='\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\';
  begin
    result:=false;
    if OptForm.BAutoSearchButton.Enabled and
       Reg.OpenKeyReadOnly(AppPaths+AExeName) and
       Reg.ValueExists('') then APathToBr:=Reg.ReadString('')
    else exit;
    result:=true;
    end;

begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if GetPathToBr('Netscape.exe',XPathToBr) then begin
      if Reg.OpenKeyReadOnly('\SOFTWARE\Netscape\Netscape Navigator') then
        if Reg.ValueExists('CurrentVersion') then XBrName:=Reg.ReadString('CurrentVersion');
      SetPathToBr('Netscape Navigator '+XBrName,XPathToBr);
      end;
    if GetPathToBr('IEXPLORE.EXE',XPathToBr) then begin
      if Reg.OpenKeyReadOnly('\SOFTWARE\Microsoft\Internet Explorer') then
        if Reg.ValueExists('Version') then XBrName:=Reg.ReadString('Version');
      SetPathToBr('Internet Explorer '+XBrName,XPathToBr);
      end;
    if GetPathToBr('Opera.exe',XPathToBr) then SetPathToBr('Opera',XPathToBr);
    if GetPathToBr('mosaic.exe',XPathToBr) then SetPathToBr('Mosaic',XPathToBr);
    if GetPathToBr('amaya.exe',XPathToBr) then SetPathToBr('Amaya',XPathToBr);
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

function RWGetActiveFile(var AFileData:TFileData):boolean;
var Reg: TRegistry;
  {--}
  function RegReadBool(AName:string; var AValue:boolean):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadBool(AName);
      result:=true;
      end;
    end;
  function RegReadInteger(AName:string; var AValue:integer):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadInteger(AName);
      result:=true;
      end;
    end;
  function RegReadString(AName:string; var AValue:string):boolean;
  begin
    result:=false;
    if Reg.ValueExists(AName) then begin
      AValue:=Reg.ReadString(AName);
      result:=true;
      end;
    end;
  {--}
begin
  result:=false;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(AppRegistryKey) and
       RegReadString('LastActiveFileName',AFileData.FileName) and
       RegReadBool('LastActiveFileWordWrap',AFileData.WordWrap) and
       RegReadInteger('LastActiveFileTopLine',AFileData.TopLine) and
       RegReadInteger('LastActiveFileCurX',AFileData.CurX) and
       RegReadInteger('LastActiveFileCurY',AFileData.CurY)
      then result:=true;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure RWSetActiveFile(AFileData:TFileData);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(AppRegistryKey,True)
    then with Reg do begin
      WriteString('LastActiveFileName',AFileData.FileName);
      WriteBool('LastActiveFileWordWrap',AFileData.WordWrap);
      WriteInteger('LastActiveFileTopLine',AFileData.TopLine);
      WriteInteger('LastActiveFileCurX',AFileData.CurX);
      WriteInteger('LastActiveFileCurY',AFileData.CurY);
      end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

function CmpShowLogo:boolean;
var Reg: TRegistry;
begin
  result:=true;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(AppRegistryKey) then
      if Reg.ValueExists('ShowLogo') then
        result:=Reg.ReadBool('ShowLogo')
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

function CmpAllow1copy:boolean;
var Reg: TRegistry;
begin
  result:=true;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(AppRegistryKey) then
      if Reg.ValueExists('Allow1copy') then
        result:=Reg.ReadBool('Allow1copy')
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure SetAllow1copy(AValue:boolean);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(AppRegistryKey,True)
    then with Reg do begin
      WriteBool('Allow1copy',AValue);
      end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure RWSetParamStr1;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(AppRegistryKey,True)
    then with Reg do begin
      WriteString('ParamStr1',ExpandFileName(GetParamStrs));
      end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

function RWGetParamStr1:string;
var Reg: TRegistry;
begin
  result:='';
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(AppRegistryKey,True) then
      if Reg.ValueExists('ShowLogo') then begin
        result:=Reg.ReadString('ParamStr1');
        Reg.WriteString('ParamStr1','');
        end;
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;
//----------------------------------------------------------
function CheckContextMenu:boolean;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    result:=Reg.KeyExists('htmlfile\Shell\HTMLCoolEdit');
  finally
    Reg.Free;
    end;
  end;

procedure SetContextMenu(AExeName:string);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('htmlfile\Shell\HTMLCoolEdit',True) then
      Reg.WriteString('','Edit with HTML&CoolEdit');
    Reg.CloseKey;
    if Reg.OpenKey('htmlfile\Shell\HTMLCoolEdit\Command',True) then
      Reg.WriteString('','"'+AExeName+'" "%1"');
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure DelContextMenu;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    Reg.DeleteKey('htmlfile\Shell\HTMLCoolEdit');
  finally
    Reg.Free;
    end;
  end;
//----------------------------------------------------------
function CheckEditWith:boolean;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    result:=Reg.KeyExists('.htm\OpenWithList\HTMLCoolEdit');
  finally
    Reg.Free;
    end;
  end;

procedure SetEditWith(AExeName:string);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('.htm\OpenWithList\HTMLCoolEdit\shell\edit\command',True) then
      Reg.WriteString('','"'+AExeName+'" "%1"');
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure DelEditWith;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    Reg.DeleteKey('.htm\OpenWithList\HTMLCoolEdit');
  finally
    Reg.Free;
    end;
  end;
//----------------------------------------------------------
function CheckDefEditor(AExeName:string):boolean;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  result:=false;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKeyReadOnly('htmlfile\shell\edit\command') then
      result:=(Reg.ReadString('')='"'+AExeName+'" "%1"');
  finally
    Reg.Free;
    end;
  end;

procedure SetDefEditor(AExeName:string);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('htmlfile\shell\edit\command',True) then
      Reg.WriteString('','"'+AExeName+'" "%1"');
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure DelDefEditor;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('htmlfile\shell\edit\command',True) then
      Reg.WriteString('','"notepad.exe" "%1"');
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;
//-----------------------------------------------------------------
function CheckViewSource(AExeName:string):boolean;
var Reg: TRegistry;
begin
  result:=false;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKeyReadOnly('SOFTWARE\Microsoft\Internet Explorer\View Source Editor\Editor Name') then
      result:=(Reg.ReadString('')=AExeNAme);
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure SetViewSource(AExeName:string);
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey('SOFTWARE\Microsoft\Internet Explorer\View Source Editor\Editor Name',True) then
      Reg.WriteString('',AExeName);
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

procedure DelViewSource;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.DeleteKey('SOFTWARE\Microsoft\Internet Explorer\View Source Editor\Editor Name');
  finally
    Reg.Free;
    end;
  end;
//-----------------------------------------------------------------
function CheckProgramMenu:boolean;
var
  Directory : String;
  MyReg     : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    Directory := MyReg.ReadString('Shell Folders','Programs','') + '\HTMLCoolEdit';
    result:=DirectoryExists(Directory);
  finally
    MyReg.Free;
    end;
  end;

procedure SetProgramMenu(AExeName:string);
var
  MyObject  : IUnknown;
  MySLink   : IShellLink;
  MyPFile   : IPersistFile;
  FileName  : String;
  Directory : String;
  WFileName : WideString;
  MyReg     : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    Directory := MyReg.ReadString('Shell Folders','Programs','') + '\HTMLCoolEdit';
    CreateDir(Directory);
    MyObject := CreateComObject(CLSID_ShellLink);
    MySLink := MyObject as IShellLink;
    MyPFile := MyObject as IPersistFile;
    FileName := AExeName;
    with MySLink do begin
      SetPath(PChar(FileName));
      SetWorkingDirectory(PChar(ExtractFilePath(FileName)));
      end;
    WFileName := Directory+'\HTMLCoolEdit.lnk';
    MyPFile.Save(PWChar(WFileName),False);
    FileName := ExtractFilePath(AExeName)+'\Readme.txt';
    with MySLink do begin
      SetPath(PChar(FileName));
      SetWorkingDirectory(PChar(ExtractFilePath(FileName)));
      end;
    WFileName := Directory+'\Readme.lnk';
    MyPFile.Save(PWChar(WFileName),False);
    FileName := ExtractFilePath(AExeName)+'\HTMLCoolEdit Home Page.url';
    with MySLink do begin
      SetPath(PChar(FileName));
      SetWorkingDirectory(PChar(ExtractFilePath(FileName)));
      end;
    WFileName := Directory+'\HTMLCoolEdit Home Page.lnk';
    MyPFile.Save(PWChar(WFileName),False);
  finally
    MyReg.Free;
    end;
  end;

procedure DelProgramMenu;
var
  Directory : String;
  MyReg     : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    Directory := MyReg.ReadString('Shell Folders','Programs','') + '\HTMLCoolEdit';
    DeleteFile(Directory+'\HTMLCoolEdit.lnk');
    DeleteFile(Directory+'\Readme.lnk');
    DeleteFile(Directory+'\HTMLCoolEdit Home Page.lnk');
    RemoveDir(Directory);
  finally
    MyReg.Free;
    end;
  end;
//-----------------------------------------------------------------
function CheckExplorer(AFolder:string):boolean;
var MyReg : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    result:=FileExists(MyReg.ReadString('Shell Folders',AFolder,'') + '\HTMLCoolEdit.lnk');
  finally
    MyReg.Free;
    end;
  end;

procedure SetExplorer(AFolder:string;AExeName:string);
var
  MyObject  : IUnknown;
  MySLink   : IShellLink;
  MyPFile   : IPersistFile;
  FileName  : String;
  WFileName : WideString;
  MyReg     : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    MyObject := CreateComObject(CLSID_ShellLink);
    MySLink := MyObject as IShellLink;
    MyPFile := MyObject as IPersistFile;
    FileName := AExeName;
    with MySLink do begin
      SetPath(PChar(FileName));
      SetWorkingDirectory(PChar(ExtractFilePath(FileName)));
      end;
    WFileName := MyReg.ReadString('Shell Folders',AFolder,'') + '\HTMLCoolEdit.lnk';
    MyPFile.Save(PWChar(WFileName),False);
  finally
    MyReg.Free;
    end;
  end;

procedure DelExplorer(AFolder:string);
var MyReg : TRegIniFile;
begin
  MyReg := TRegIniFile.Create('Software\MicroSoft\Windows\CurrentVersion\Explorer');
  try
    DeleteFile(MyReg.ReadString('Shell Folders',AFolder,'')+'\HTMLCoolEdit.lnk');
  finally
    MyReg.Free;
    end;
  end;

procedure SetURLHistoryBox;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKeyReadOnly(AppRegistryKey) then
      if Reg.ValueExists('URLHistoryBox') then
        OpenWebForm.URLHistoryBox.Items.Text:=Reg.ReadString('URLHistoryBox');
  finally
    Reg.CloseKey;
    Reg.Free;
    end;
  end;

end.
