{*******************************************************}
{                                                       }
{                     HTMLCoolEdit                      }
{                                                       }
{           Copyright (c) 1999-2000 PROFOX              }
{                                                       }
{*******************************************************}

unit ReplDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, SyntaxEd;

type
  TEdReplDlg = class(TForm)
    Panel1: TPanel;
    OKButton: TButton;
    CancelButton: TButton;
    Panel2: TPanel;
    Label1: TLabel;
    FindHistoryBox: TComboBox;
    OptionsGroupBox: TGroupBox;
    UseCaseBox: TCheckBox;
    WholeWordBox: TCheckBox;
    RegExBox: TCheckBox;
    DirectionGroupBox: TGroupBox;
    FwdDirection: TRadioButton;
    BackDirection: TRadioButton;
    ReplaceAllButton: TButton;
    Label2: TLabel;
    ReplHistoryBox: TComboBox;
    PromptOnReplBox: TCheckBox;
    ScopeGroupBox: TGroupBox;
    GlobalScope: TRadioButton;
    SelectedScope: TRadioButton;
    OriginGroupBox: TGroupBox;
    CursorOrigin: TRadioButton;
    EntireOrigin: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OKButtonClick(Sender: TObject);
    procedure ReplaceAllButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FindHistoryBoxChange(Sender: TObject);
    procedure RegExBoxClick(Sender: TObject);
    procedure SelectedScopeClick(Sender: TObject);
    procedure GlobalScopeClick(Sender: TObject);
  private
    { Private declarations }
    FFindText          : string;
    FReplaceText       : string;
    FReplaceAll        : Boolean;
    FStartPoint        : longint;
    FOriginalSelection : TChRange;
    FModalResult       : integer;
    FLastFindText              : string;
    procedure UpdateOptionBoxes;
    procedure ReadOptionsBoxes;
  public
    { Public declarations }
    FFindOpt           : longint;
    function Execute(FindText, ReplaceText: string; Options: Integer):integer;
    procedure DoReplace(anEditor: TSyntaxMemo);
  end;

var
  EdReplDlg: TEdReplDlg;

implementation

{$R *.DFM}

{ TEdReplDlg }

const
  udREPLACEALL = UDS_USER + 1;

function TEdReplDlg.Execute(FindText, ReplaceText: string; Options: Integer):integer;
begin
  FFindText := FindText;
  FReplaceText := ReplaceText;
  FFindOpt  := Options;
  FLastFindText  := '';
  ShowModal;
  result := FModalResult;
  end;

procedure TEdReplDlg.FormShow(Sender: TObject);
begin
  if (FFindText <> '') then begin
    if FindHistoryBox.Items.IndexOf(FFindText) = -1 then FindHistoryBox.Items.Add(FFindText);
    FindHistoryBox.ItemIndex := FindHistoryBox.Items.IndexOf(FFindText);
    end;
  if (FReplaceText <> '') then begin
    if ReplHistoryBox.Items.IndexOf(FReplaceText) = -1 then ReplHistoryBox.Items.Add(FReplaceText);
    ReplHistoryBox.ItemIndex := ReplHistoryBox.Items.IndexOf(FReplaceText);
    end;
  UpdateOptionBoxes;
  FindHistoryBoxChange(Self);
  FindHistoryBox.Setfocus;
  end;

procedure TEdReplDlg.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then begin
    FModalResult := mrCancel;
    Close;
    end;
  end;

procedure TEdReplDlg.DoReplace(anEditor: TSyntaxMemo);
var
  aSel    : TChRange;
  rText   : string;
  MemPromt   : Boolean;
  CntReplace : longint;
  XFirstRepl : Boolean;
begin
  MemPromt:=PromptOnReplBox.Checked;
  anEditor.Perform(SEM_SELECTION, eaGET, longint(@FOriginalSelection));
  FOriginalSelection := Normalise(FOriginalSelection);
  if GlobalScope.Checked then
    if EntireOrigin.Checked then
      if FwdDirection.Checked
        then FStartPoint := 0
        else FStartPoint := Length(anEditor.Lines.Text)-1
    else FStartPoint := FOriginalSelection.chStart
  else //Selected text
    if EntireOrigin.Checked then
      if FwdDirection.Checked
        then FStartPoint := FOriginalSelection.chStart //Forward
        else FStartPoint := FOriginalSelection.chEnd   //Backward
    else FStartPoint := FOriginalSelection.chStart; //From cursor
  with anEditor do begin
    if FLastFindText<>'' then
      if (UseCaseBox.Checked and (SelText = FLastFindText))
         or ((not UseCaseBox.Checked) and (Lowercase(SelText) = Lowercase(FLastFindText)))
      then
        if FwdDirection.Checked
          then FStartPoint := FStartPoint + 1
          else FStartPoint := FStartPoint - 1;
    end;
  if FReplaceAll then begin
    CntReplace:=0;
    with anEditor do begin
      XFirstRepl:=true;
      if not PromptOnReplBox.Checked then Lines.BeginUpdate;
      repeat
        anEditor.Perform(EM_SETSEL, FStartPoint, FStartPoint);
        if (not FindReplace(FindHistoryBox.Text, ReplHistoryBox.Text, FStartPoint, rText, FFindOpt))
            or (SelectedScope.Checked and (
              (FStartPoint < FOriginalSelection.chStart)
              or (FStartPoint+length(FindHistoryBox.Text) > FOriginalSelection.chEnd)))
          then begin
            //SelLength:=0;
            if PromptOnReplBox.Checked or (CntReplace=0) then
              MessageDlg(format('"%s" not found', [FindHistoryBox.Text]),mtInformation,[mbOk],0);
            break;
            end
          else begin
            FLastFindText := SelText;
            if PromptOnReplBox.Checked then
              case MessageDlg(format('Replace this instance of "%s" with "%s" ?', [FindHistoryBox.Text, rText]), mtConfirmation, [mbYES, mbNO, mbCANCEL, mbAll], 0) of
                mrYES    : begin
                  if XFirstRepl then begin
                    UndoTransBegin(udREPLACEALL);
                    XFirstRepl:=false;
                    end;
                  Perform(SEM_SELECTION,  eaGET, longint(@aSel));
                  aSel := Normalise(aSel);
                  Perform(SEM_REPLACESEL, euNODRAW, longint(PChar(rText)));
                  if SelectedScope.Checked then
                    inc(FOriginalSelection.chEnd, length(rText) - (aSel.chEnd - aSel.chStart));
                  Perform(SEM_SELECTION,  eaGET, longint(@aSel));
                  aSel := Normalise(aSel);
                  if FwdDirection.Checked
                    then FStartPoint := aSel.chEnd
                    else FStartPoint := aSel.chStart-1;
                  end;
                mrNo     : begin
                  Perform(SEM_SELECTION,  eaGET, longint(@aSel));
                  aSel := Normalise(aSel);
                  if FwdDirection.Checked
                    then FStartPoint := aSel.chEnd
                    else FStartPoint := aSel.chStart; //???
                  end;
                mrCancel : break;
                mrAll    : begin
                  PromptOnReplBox.Checked:=false;
                  Lines.BeginUpdate;
                  Continue;
                  end;
                end
            else begin
             if XFirstRepl then begin
               UndoTransBegin(udREPLACEALL);
               XFirstRepl:=false;
               end;
              Perform(SEM_SELECTION,  eaGET, longint(@aSel));
              aSel := Normalise(aSel);
              Perform(SEM_REPLACESEL, euNODRAW, longint(PChar(rText)));
              inc(CntReplace);
              if SelectedScope.Checked then inc(FOriginalSelection.chEnd, length(rText) - (aSel.chEnd - aSel.chStart));
              Perform(SEM_SELECTION,  eaGET, longint(@aSel));
              aSel := Normalise(aSel);
              if FwdDirection.Checked
                then FStartPoint := aSel.chEnd
                else FStartPoint := aSel.chStart-1;
              end;
            end;
        until false;
      if not PromptOnReplBox.Checked then Lines.EndUpdate;
      if not XFirstRepl then UndoTransEnd(udREPLACEALL);
      UndoTransEnd(udREPLACEALL);
      Perform(SEM_SELECTION, eaGET, longint(@aSel));
      Perform(SEM_SELECTION, eaSET or euDRAW, longint(@aSel));
      end;
    end
  else begin
    with anEditor do begin
      anEditor.Perform(EM_SETSEL, FStartPoint, FStartPoint);
      if (not FindReplace(FindHistoryBox.Text, ReplHistoryBox.Text, FStartPoint, rText, FFindOpt))
            or (SelectedScope.Checked and (
              (FStartPoint < FOriginalSelection.chStart)
              or (FStartPoint+length(FindHistoryBox.Text) > FOriginalSelection.chEnd)))
      then begin
        //SelLength:=0;
        MessageDlg(format('"%s" not found', [FindHistoryBox.Text]),mtInformation,[mbOk],0);
      end else begin
        FLastFindText := SelText;
        if PromptOnReplBox.Checked then
          case MessageDlg(format('Replace this instance of "%s" with "%s" ?', [FindHistoryBox.Text, rText]), mtConfirmation, [mbYES, mbNO, mbCANCEL, mbAll], 0) of
            mrYES    : Perform(SEM_REPLACESEL, euDRAW, longint(PChar(rText)));
            mrNo     : ;
            mrCancel : exit;
            mrAll    : begin
              FReplaceAll:=true;
              DoReplace(anEditor);
              FReplaceAll:=false;
              end;
            end
          else Perform(SEM_REPLACESEL, euDRAW, longint(PChar(rText)));
        end;
      end;
    end;
  PromptOnReplBox.Checked:=MemPromt;
  end;

procedure TEdReplDlg.OKButtonClick(Sender: TObject);
begin
  ReadOptionsBoxes;
  FReplaceAll:=false;
  if FindHistoryBox.Items.IndexOf(FindHistoryBox.Text) = -1 then
    FindHistoryBox.Items.Insert(0, FindHistoryBox.Text);
  if ReplHistoryBox.Items.IndexOf(ReplHistoryBox.Text) = -1 then
    ReplHistoryBox.Items.Insert(0, ReplHistoryBox.Text);
  FModalResult := mrOk;
  Close;
  end;

procedure TEdReplDlg.ReadOptionsBoxes;
begin
  FFindOpt := 0;
  if UseCaseBox.Checked    then FFindOpt := FFindOpt or ft_MATCHCASE;
  if WholeWordBox.Checked  then FFindOpt := FFindOpt or ft_WHOLEWORD;
  if RegExBox.Checked      then FFindOpt := FFindOpt or ft_REGEXPR;
  if BackDirection.Checked then FFindOpt := FFindOpt or ft_REVERSE;
  end;

procedure TEdReplDlg.UpdateOptionBoxes;
begin
  UseCaseBox.Checked    := FFindOpt and ft_MATCHCASE <> 0;
  WholeWordBox.Checked  := FFindOpt and ft_WHOLEWORD <> 0;
  RegExBox.Checked      := FFindOpt and ft_REGEXPR   <> 0;
  FwdDirection.Checked  := FFindOpt and ft_REVERSE   =  0;
  BackDirection.Checked := not (FwdDirection.Checked);
  end;

procedure TEdReplDlg.ReplaceAllButtonClick(Sender: TObject);
begin
  // Replace all occurences...
  ReadOptionsBoxes;
  FReplaceAll:=true;
  if FindHistoryBox.Items.IndexOf(FindHistoryBox.Text) = -1 then
    FindHistoryBox.Items.Insert(0, FindHistoryBox.Text);
  if ReplHistoryBox.Items.IndexOf(ReplHistoryBox.Text) = -1 then
    ReplHistoryBox.Items.Insert(0, ReplHistoryBox.Text);
  FModalResult := mrAll;
  Close;
  end;

procedure TEdReplDlg.CancelButtonClick(Sender: TObject);
begin
  FModalResult := mrCancel;
  Close;
  end;

procedure TEdReplDlg.FindHistoryBoxChange(Sender: TObject);
begin
  OKButton.Enabled         := FindHistoryBox.Text <> '';
  ReplaceAllButton.Enabled := OKButton.Enabled;
  end;

procedure TEdReplDlg.RegExBoxClick(Sender: TObject);
begin
  UseCaseBox.Enabled:=not RegExBox.Checked;
  WholeWordBox.Enabled:=not RegExBox.Checked;
  end;

procedure TEdReplDlg.GlobalScopeClick(Sender: TObject);
begin
  CursorOrigin.Enabled:=true;
  end;

procedure TEdReplDlg.SelectedScopeClick(Sender: TObject);
begin
  EntireOrigin.Checked:=true;
  CursorOrigin.Enabled:=false;
  end;

end.
